/* abstractcollection.vala
 *
 * Copyright (C) 2007  Jürg Billeter
 * Copyright (C) 2009  Didier Villevalois
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes' Villevalois <ptitjes@free.fr>
 */

#include <glib.h>
#include <glib-object.h>


#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))

typedef struct _GeeIterable GeeIterable;
typedef struct _GeeIterableIface GeeIterableIface;

#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))

typedef struct _GeeIterator GeeIterator;
typedef struct _GeeIteratorIface GeeIteratorIface;

#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))

typedef struct _GeeCollection GeeCollection;
typedef struct _GeeCollectionIface GeeCollectionIface;

#define GEE_TYPE_ABSTRACT_COLLECTION (gee_abstract_collection_get_type ())
#define GEE_ABSTRACT_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollection))
#define GEE_ABSTRACT_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionClass))
#define GEE_IS_ABSTRACT_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_COLLECTION))
#define GEE_IS_ABSTRACT_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_COLLECTION))
#define GEE_ABSTRACT_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionClass))

typedef struct _GeeAbstractCollection GeeAbstractCollection;
typedef struct _GeeAbstractCollectionClass GeeAbstractCollectionClass;
typedef struct _GeeAbstractCollectionPrivate GeeAbstractCollectionPrivate;

/**
 * Implemented by classes that support a simple iteration over instances of the
 * collection.
 */
struct _GeeIteratorIface {
	GTypeInterface parent_iface;
	gboolean (*next) (GeeIterator* self);
	gpointer (*get) (GeeIterator* self);
};

/**
 * Implemented by classes that support a simple iteration over instances of the
 * collection.
 */
struct _GeeIterableIface {
	GTypeInterface parent_iface;
	GeeIterator* (*iterator) (GeeIterable* self);
	GType (*get_element_type) (GeeIterable* self);
};

/**
 * Serves as the base interface for implementing collection classes. Defines
 * size, iteration, and modification methods.
 */
struct _GeeCollectionIface {
	GTypeInterface parent_iface;
	gboolean (*contains) (GeeCollection* self, gconstpointer item);
	gboolean (*add) (GeeCollection* self, gconstpointer item);
	gboolean (*remove) (GeeCollection* self, gconstpointer item);
	void (*clear) (GeeCollection* self);
	gboolean (*add_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*contains_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*remove_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*retain_all) (GeeCollection* self, GeeCollection* collection);
	gpointer* (*to_array) (GeeCollection* self, int* result_length1);
	gint (*get_size) (GeeCollection* self);
	gboolean (*get_is_empty) (GeeCollection* self);
};

/**
 * Serves as the base class for implementing collection classes.
 */
struct _GeeAbstractCollection {
	GObject parent_instance;
	GeeAbstractCollectionPrivate * priv;
};

struct _GeeAbstractCollectionClass {
	GObjectClass parent_class;
	gboolean (*contains) (GeeAbstractCollection* self, gconstpointer item);
	gboolean (*add) (GeeAbstractCollection* self, gconstpointer item);
	gboolean (*remove) (GeeAbstractCollection* self, gconstpointer item);
	void (*clear) (GeeAbstractCollection* self);
	gpointer* (*to_array) (GeeAbstractCollection* self, int* result_length1);
	gboolean (*add_all) (GeeAbstractCollection* self, GeeCollection* collection);
	gboolean (*contains_all) (GeeAbstractCollection* self, GeeCollection* collection);
	gboolean (*remove_all) (GeeAbstractCollection* self, GeeCollection* collection);
	gboolean (*retain_all) (GeeAbstractCollection* self, GeeCollection* collection);
	GeeIterator* (*iterator) (GeeAbstractCollection* self);
	gint (*get_size) (GeeAbstractCollection* self);
	gboolean (*get_is_empty) (GeeAbstractCollection* self);
};

struct _GeeAbstractCollectionPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};


static gpointer gee_abstract_collection_parent_class = NULL;
static GeeIterableIface* gee_abstract_collection_gee_iterable_parent_iface = NULL;
static GeeCollectionIface* gee_abstract_collection_gee_collection_parent_iface = NULL;

GType gee_iterator_get_type (void);
GType gee_iterable_get_type (void);
GType gee_collection_get_type (void);
GType gee_abstract_collection_get_type (void);
#define GEE_ABSTRACT_COLLECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionPrivate))
enum  {
	GEE_ABSTRACT_COLLECTION_DUMMY_PROPERTY,
	GEE_ABSTRACT_COLLECTION_SIZE,
	GEE_ABSTRACT_COLLECTION_IS_EMPTY,
	GEE_ABSTRACT_COLLECTION_ELEMENT_TYPE,
	GEE_ABSTRACT_COLLECTION_G_TYPE,
	GEE_ABSTRACT_COLLECTION_G_DUP_FUNC,
	GEE_ABSTRACT_COLLECTION_G_DESTROY_FUNC
};
gboolean gee_abstract_collection_contains (GeeAbstractCollection* self, gconstpointer item);
static gboolean gee_abstract_collection_real_contains (GeeAbstractCollection* self, gconstpointer item);
gboolean gee_abstract_collection_add (GeeAbstractCollection* self, gconstpointer item);
static gboolean gee_abstract_collection_real_add (GeeAbstractCollection* self, gconstpointer item);
gboolean gee_abstract_collection_remove (GeeAbstractCollection* self, gconstpointer item);
static gboolean gee_abstract_collection_real_remove (GeeAbstractCollection* self, gconstpointer item);
void gee_abstract_collection_clear (GeeAbstractCollection* self);
static void gee_abstract_collection_real_clear (GeeAbstractCollection* self);
gint gee_collection_get_size (GeeCollection* self);
GeeIterator* gee_abstract_collection_iterator (GeeAbstractCollection* self);
gboolean gee_iterator_next (GeeIterator* self);
gpointer gee_iterator_get (GeeIterator* self);
gpointer* gee_abstract_collection_to_array (GeeAbstractCollection* self, int* result_length1);
static gpointer* gee_abstract_collection_real_to_array (GeeAbstractCollection* self, int* result_length1);
gboolean gee_collection_get_is_empty (GeeCollection* self);
GeeIterator* gee_iterable_iterator (GeeIterable* self);
gboolean gee_abstract_collection_add_all (GeeAbstractCollection* self, GeeCollection* collection);
static gboolean gee_abstract_collection_real_add_all (GeeAbstractCollection* self, GeeCollection* collection);
gboolean gee_abstract_collection_contains_all (GeeAbstractCollection* self, GeeCollection* collection);
static gboolean gee_abstract_collection_real_contains_all (GeeAbstractCollection* self, GeeCollection* collection);
gboolean gee_abstract_collection_remove_all (GeeAbstractCollection* self, GeeCollection* collection);
static gboolean gee_abstract_collection_real_remove_all (GeeAbstractCollection* self, GeeCollection* collection);
gboolean gee_collection_contains (GeeCollection* self, gconstpointer item);
gboolean gee_abstract_collection_retain_all (GeeAbstractCollection* self, GeeCollection* collection);
static gboolean gee_abstract_collection_real_retain_all (GeeAbstractCollection* self, GeeCollection* collection);
static GeeIterator* gee_abstract_collection_real_iterator (GeeAbstractCollection* self);
gint gee_abstract_collection_get_size (GeeAbstractCollection* self);
gboolean gee_abstract_collection_get_is_empty (GeeAbstractCollection* self);
static void gee_abstract_collection_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void gee_abstract_collection_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);



static gboolean gee_abstract_collection_real_contains (GeeAbstractCollection* self, gconstpointer item) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_collection_contains'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return FALSE;
}


gboolean gee_abstract_collection_contains (GeeAbstractCollection* self, gconstpointer item) {
	return GEE_ABSTRACT_COLLECTION_GET_CLASS (self)->contains (self, item);
}


static gboolean gee_abstract_collection_real_add (GeeAbstractCollection* self, gconstpointer item) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_collection_add'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return FALSE;
}


gboolean gee_abstract_collection_add (GeeAbstractCollection* self, gconstpointer item) {
	return GEE_ABSTRACT_COLLECTION_GET_CLASS (self)->add (self, item);
}


static gboolean gee_abstract_collection_real_remove (GeeAbstractCollection* self, gconstpointer item) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_collection_remove'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return FALSE;
}


gboolean gee_abstract_collection_remove (GeeAbstractCollection* self, gconstpointer item) {
	return GEE_ABSTRACT_COLLECTION_GET_CLASS (self)->remove (self, item);
}


static void gee_abstract_collection_real_clear (GeeAbstractCollection* self) {
	g_return_if_fail (self != NULL);
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_collection_clear'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return;
}


void gee_abstract_collection_clear (GeeAbstractCollection* self) {
	GEE_ABSTRACT_COLLECTION_GET_CLASS (self)->clear (self);
}


static gpointer* gee_abstract_collection_real_to_array (GeeAbstractCollection* self, int* result_length1) {
	gpointer* result;
	gpointer* _tmp1_;
	gint array_size;
	gint array_length1;
	gint _tmp0_;
	gpointer* array;
	gint index;
	gpointer* _tmp5_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp1_ = NULL;
	array = (_tmp1_ = g_new0 (gpointer, _tmp0_ = gee_collection_get_size ((GeeCollection*) self)), array_length1 = _tmp0_, array_size = array_length1, _tmp1_);
	index = 0;
	{
		GeeIterator* _element_it;
		_element_it = gee_abstract_collection_iterator (self);
		while (TRUE) {
			gpointer element;
			gpointer _tmp4_;
			gpointer *_tmp3_;
			gconstpointer _tmp2_;
			if (!gee_iterator_next (_element_it)) {
				break;
			}
			element = gee_iterator_get (_element_it);
			_tmp4_ = NULL;
			_tmp2_ = NULL;
			_tmp3_ = &array[index++];
			(*_tmp3_) = (_tmp4_ = (_tmp2_ = element, ((_tmp2_ == NULL) || (self->priv->g_dup_func == NULL)) ? ((gpointer) _tmp2_) : self->priv->g_dup_func ((gpointer) _tmp2_)), (((*_tmp3_) == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : ((*_tmp3_) = (self->priv->g_destroy_func ((*_tmp3_)), NULL)), _tmp4_);
			((element == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (element = (self->priv->g_destroy_func (element), NULL));
		}
		(_element_it == NULL) ? NULL : (_element_it = (g_object_unref (_element_it), NULL));
	}
	_tmp5_ = NULL;
	result = (_tmp5_ = array, *result_length1 = array_length1, _tmp5_);
	return result;
	array = (_vala_array_free (array, array_length1, (GDestroyNotify) self->priv->g_destroy_func), NULL);
}


gpointer* gee_abstract_collection_to_array (GeeAbstractCollection* self, int* result_length1) {
	return GEE_ABSTRACT_COLLECTION_GET_CLASS (self)->to_array (self, result_length1);
}


static gboolean gee_abstract_collection_real_add_all (GeeAbstractCollection* self, GeeCollection* collection) {
	gboolean result;
	gboolean changed;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (collection != NULL, FALSE);
	if (gee_collection_get_is_empty (collection)) {
		result = FALSE;
		return result;
	}
	changed = FALSE;
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator ((GeeIterable*) collection);
		while (TRUE) {
			gpointer item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = gee_iterator_get (_item_it);
			changed = changed | gee_abstract_collection_add (self, item);
			((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	result = changed;
	return result;
}


gboolean gee_abstract_collection_add_all (GeeAbstractCollection* self, GeeCollection* collection) {
	return GEE_ABSTRACT_COLLECTION_GET_CLASS (self)->add_all (self, collection);
}


static gboolean gee_abstract_collection_real_contains_all (GeeAbstractCollection* self, GeeCollection* collection) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (collection != NULL, FALSE);
	if (gee_collection_get_size (collection) > gee_collection_get_size ((GeeCollection*) self)) {
		result = FALSE;
		return result;
	}
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator ((GeeIterable*) collection);
		while (TRUE) {
			gpointer item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = gee_iterator_get (_item_it);
			if (!gee_abstract_collection_contains (self, item)) {
				result = FALSE;
				((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
				(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
				return result;
			}
			((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	result = TRUE;
	return result;
}


gboolean gee_abstract_collection_contains_all (GeeAbstractCollection* self, GeeCollection* collection) {
	return GEE_ABSTRACT_COLLECTION_GET_CLASS (self)->contains_all (self, collection);
}


static gboolean gee_abstract_collection_real_remove_all (GeeAbstractCollection* self, GeeCollection* collection) {
	gboolean result;
	gboolean changed;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (collection != NULL, FALSE);
	changed = FALSE;
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator ((GeeIterable*) collection);
		while (TRUE) {
			gpointer item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = gee_iterator_get (_item_it);
			changed = changed | gee_abstract_collection_remove (self, item);
			((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
	result = changed;
	return result;
}


gboolean gee_abstract_collection_remove_all (GeeAbstractCollection* self, GeeCollection* collection) {
	return GEE_ABSTRACT_COLLECTION_GET_CLASS (self)->remove_all (self, collection);
}


static gboolean gee_abstract_collection_real_retain_all (GeeAbstractCollection* self, GeeCollection* collection) {
	gboolean result;
	gboolean changed;
	gpointer* _tmp1_;
	gint items_size;
	gint items_length1;
	gint _tmp0_;
	gpointer* items;
	gint size_of_items;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (collection != NULL, FALSE);
	changed = FALSE;
	_tmp1_ = NULL;
	items = (_tmp1_ = gee_abstract_collection_to_array (self, &_tmp0_), items_length1 = _tmp0_, items_size = items_length1, _tmp1_);
	size_of_items = gee_collection_get_size ((GeeCollection*) self);
	{
		gint index;
		index = 0;
		{
			gboolean _tmp2_;
			_tmp2_ = TRUE;
			while (TRUE) {
				if (!_tmp2_) {
					index++;
				}
				_tmp2_ = FALSE;
				if (!(index < size_of_items)) {
					break;
				}
				if (!gee_collection_contains (collection, items[index])) {
					changed = changed | gee_abstract_collection_remove (self, items[index]);
				}
			}
		}
	}
	result = changed;
	items = (_vala_array_free (items, items_length1, (GDestroyNotify) self->priv->g_destroy_func), NULL);
	return result;
}


gboolean gee_abstract_collection_retain_all (GeeAbstractCollection* self, GeeCollection* collection) {
	return GEE_ABSTRACT_COLLECTION_GET_CLASS (self)->retain_all (self, collection);
}


static GeeIterator* gee_abstract_collection_real_iterator (GeeAbstractCollection* self) {
	g_return_val_if_fail (self != NULL, NULL);
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_collection_iterator'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


GeeIterator* gee_abstract_collection_iterator (GeeAbstractCollection* self) {
	return GEE_ABSTRACT_COLLECTION_GET_CLASS (self)->iterator (self);
}


gint gee_abstract_collection_get_size (GeeAbstractCollection* self) {
	return GEE_ABSTRACT_COLLECTION_GET_CLASS (self)->get_size (self);
}


gboolean gee_abstract_collection_get_is_empty (GeeAbstractCollection* self) {
	return GEE_ABSTRACT_COLLECTION_GET_CLASS (self)->get_is_empty (self);
}


static gboolean gee_abstract_collection_real_get_is_empty (GeeAbstractCollection* base) {
	gboolean result;
	GeeAbstractCollection* self;
	self = base;
	result = gee_collection_get_size ((GeeCollection*) self) == 0;
	return result;
}


static GType gee_abstract_collection_real_get_element_type (GeeIterable* base) {
	GType result;
	GeeAbstractCollection* self;
	self = (GeeAbstractCollection*) base;
	result = self->priv->g_type;
	return result;
}


static void gee_abstract_collection_class_init (GeeAbstractCollectionClass * klass) {
	gee_abstract_collection_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeAbstractCollectionPrivate));
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->contains = gee_abstract_collection_real_contains;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->add = gee_abstract_collection_real_add;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->remove = gee_abstract_collection_real_remove;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->clear = gee_abstract_collection_real_clear;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->to_array = gee_abstract_collection_real_to_array;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->add_all = gee_abstract_collection_real_add_all;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->contains_all = gee_abstract_collection_real_contains_all;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->remove_all = gee_abstract_collection_real_remove_all;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->retain_all = gee_abstract_collection_real_retain_all;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->iterator = gee_abstract_collection_real_iterator;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_is_empty = gee_abstract_collection_real_get_is_empty;
	G_OBJECT_CLASS (klass)->get_property = gee_abstract_collection_get_property;
	G_OBJECT_CLASS (klass)->set_property = gee_abstract_collection_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_COLLECTION_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_COLLECTION_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_COLLECTION_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_COLLECTION_SIZE, "size");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_COLLECTION_IS_EMPTY, "is-empty");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_COLLECTION_ELEMENT_TYPE, "element-type");
}


static void gee_abstract_collection_gee_iterable_interface_init (GeeIterableIface * iface) {
	gee_abstract_collection_gee_iterable_parent_iface = g_type_interface_peek_parent (iface);
	iface->iterator = (GeeIterator* (*)(GeeIterable*)) gee_abstract_collection_iterator;
	iface->get_element_type = gee_abstract_collection_real_get_element_type;
}


static void gee_abstract_collection_gee_collection_interface_init (GeeCollectionIface * iface) {
	gee_abstract_collection_gee_collection_parent_iface = g_type_interface_peek_parent (iface);
	iface->contains = (gboolean (*)(GeeCollection* ,gconstpointer)) gee_abstract_collection_contains;
	iface->add = (gboolean (*)(GeeCollection* ,gconstpointer)) gee_abstract_collection_add;
	iface->remove = (gboolean (*)(GeeCollection* ,gconstpointer)) gee_abstract_collection_remove;
	iface->clear = (void (*)(GeeCollection*)) gee_abstract_collection_clear;
	iface->to_array = (gpointer* (*)(GeeCollection* ,int*)) gee_abstract_collection_to_array;
	iface->add_all = (gboolean (*)(GeeCollection* ,GeeCollection*)) gee_abstract_collection_add_all;
	iface->contains_all = (gboolean (*)(GeeCollection* ,GeeCollection*)) gee_abstract_collection_contains_all;
	iface->remove_all = (gboolean (*)(GeeCollection* ,GeeCollection*)) gee_abstract_collection_remove_all;
	iface->retain_all = (gboolean (*)(GeeCollection* ,GeeCollection*)) gee_abstract_collection_retain_all;
	iface->get_size = gee_abstract_collection_get_size;
	iface->get_is_empty = gee_abstract_collection_get_is_empty;
}


static void gee_abstract_collection_instance_init (GeeAbstractCollection * self) {
	self->priv = GEE_ABSTRACT_COLLECTION_GET_PRIVATE (self);
}


GType gee_abstract_collection_get_type (void) {
	static GType gee_abstract_collection_type_id = 0;
	if (gee_abstract_collection_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeAbstractCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_abstract_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeAbstractCollection), 0, (GInstanceInitFunc) gee_abstract_collection_instance_init, NULL };
		static const GInterfaceInfo gee_iterable_info = { (GInterfaceInitFunc) gee_abstract_collection_gee_iterable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_collection_info = { (GInterfaceInitFunc) gee_abstract_collection_gee_collection_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		gee_abstract_collection_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeAbstractCollection", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_type_add_interface_static (gee_abstract_collection_type_id, GEE_TYPE_ITERABLE, &gee_iterable_info);
		g_type_add_interface_static (gee_abstract_collection_type_id, GEE_TYPE_COLLECTION, &gee_collection_info);
	}
	return gee_abstract_collection_type_id;
}


static void gee_abstract_collection_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeAbstractCollection * self;
	gpointer boxed;
	self = GEE_ABSTRACT_COLLECTION (object);
	switch (property_id) {
		case GEE_ABSTRACT_COLLECTION_IS_EMPTY:
		g_value_set_boolean (value, gee_abstract_collection_get_is_empty (self));
		break;
		case GEE_ABSTRACT_COLLECTION_ELEMENT_TYPE:
		g_value_set_gtype (value, gee_iterable_get_element_type ((GeeIterable*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_abstract_collection_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeAbstractCollection * self;
	self = GEE_ABSTRACT_COLLECTION (object);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
		case GEE_ABSTRACT_COLLECTION_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_ABSTRACT_COLLECTION_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_COLLECTION_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}




